/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/


#include <stdlib.h>
#include "ER-EPM0213-2B.h"

Epd::~Epd() {
};

Epd::Epd() {
    reset_pin = RST_PIN;
    dc_pin = DC_PIN;
    cs_pin = CS_PIN;
    busy_pin = BUSY_PIN;
    width = EPD_WIDTH;
    height = EPD_HEIGHT;
};




int Epd::Init(void) {
    /* this calls the peripheral hardware interface, see epdif */
    if (IfInit() != 0) {
        return -1;
    }
    /* EPD hardware init start */
    Reset();
 
  
    SendCommand(0x00);       
    SendData(0xFF);//You can change the scanning direction, FF -> FB F3 F7 to rotate and flip left and right and up and down

		
    SendCommand(0x01); 
    SendData(0x03);    
    SendData(0x00);    
    SendData(0x3F);    
    SendData(0x3F);    
    SendData(0x03);    

    SendCommand(0x03);       
    SendData(0x00);    

    SendCommand(0x06);       
    SendData(0x27);    
    SendData(0x27);    
    SendData(0x2F);    

    SendCommand(0x30);       
    SendData(0x09);   

    SendCommand(0x60);       
    SendData (0x22);   

    SendCommand(0x82);        // vcom
    SendData(0x00);      // 

    SendCommand(0xE3);        
    SendData(0x00);    

    SendCommand(0x41);        
    SendData(0x00);   
		
    SendCommand(0x61);    
    SendData(0x80);     
    SendData(0x00);   
    SendData(0xFA);  
  
    SendCommand(0x65);        
    SendData(0x00);    
    SendData(0x00);    
    SendData(0x00);    

    SendCommand(0X50);      
    SendData(0xB7); 
  
    
    /* EPD hardware init end */
    return 0;

}

/**
 *  @brief: basic function for sending commands
 */
void Epd::SendCommand(unsigned char command) {
    DigitalWrite(dc_pin, LOW);
    SpiTransfer(command);
}

/**
 *  @brief: basic function for sending data
 */
void Epd::SendData(unsigned char data) {
    DigitalWrite(dc_pin, HIGH);
    SpiTransfer(data);
}

/**
 *  @brief: Wait until the busy_pin goes HIGH
 */
void Epd::WaitUntilIdle(void) {
    while(DigitalRead(busy_pin) == 0) {      //0: busy, 1: idle
        DelayMs(100);
    }      
}

/**
 *  @brief: module reset. 
 *          often used to awaken the module in deep sleep, 
 *          see Epd::Sleep();
 */
void Epd::Reset(void) {
    DigitalWrite(reset_pin, LOW);
    DelayMs(200);
    DigitalWrite(reset_pin, HIGH);
    DelayMs(200);   
}



/**
 *  @brief: transmit partial data to the red part of SRAM
 */
void Epd::SetPartialWindow(const unsigned char* buffer_red, int x, int y, int w, int l) {
    SendCommand(PARTIAL_IN);
    SendCommand(PARTIAL_WINDOW);
    SendData(x & 0xf8);     // x should be the multiple of 8, the last 3 bit will always be ignored
    SendData(((x & 0xf8) + w  - 1) | 0x07);
    SendData(y >> 8);        
    SendData(y & 0xff);
    SendData((y + l - 1) >> 8);        
    SendData((y + l - 1) & 0xff);
    SendData(0x01);         // Gates scan both inside and outside of the partial window. (default) 
    DelayMs(2);
    SendCommand(DATA_START_TRANSMISSION_2);
    if (buffer_red != NULL) {
        for(int i = 0; i < w  / 8 * l; i++) {
            SendData(buffer_red[i]);  
        }  
    } else {
        for(int i = 0; i < w  / 8 * l; i++) {
            SendData(0x00);  
        }  
    }
    DelayMs(2);
    SendCommand(PARTIAL_OUT);  
}




/**
 * @brief: refresh and displays the frame
 */
void Epd::DisplayFrame(const unsigned char* frame_buffer_black) {
 /*   if (frame_buffer_black != NULL) {
        SendCommand(DATA_START_TRANSMISSION_1);
        DelayMs(2);
        for (unsigned int i = 0; i < this->width * this->height / 8; i++) {
             SendData(0x00);
        }
        DelayMs(2);
    }*/
    if (frame_buffer_black != NULL) {
        SendCommand(DATA_START_TRANSMISSION_2);
        DelayMs(2);
        for (unsigned int i = 0; i < this->width * this->height / 8; i++) {
            SendData(pgm_read_byte(&frame_buffer_black[i]));
        }
        DelayMs(2);
    }
    lut_GC(); 	
    DisplayFrame();
}

/**
 * @brief: clear the frame data from the SRAM, this won't refresh the display
 */
void Epd::ClearFrame(void) {


    
    SendCommand(DATA_START_TRANSMISSION_2);           
    DelayMs(2);
    for(unsigned int i = 0; i < width * height / 8; i++) {
        SendData(0xff);  
    }  
    DelayMs(2);
    lut_GC(); 
    DisplayFrame();    	
}

/**
 * @brief: This displays the frame data from SRAM
 */
void Epd::DisplayFrame(void) {
   // SendCommand(DISPLAY_REFRESH); 
   SendCommand(AUTO_SEQUENCE);      
    SendData(0xA5);  
    WaitUntilIdle();
}

/**
 * @brief: After this command is transmitted, the chip would enter the deep-sleep mode to save power. 
 *         The deep sleep mode would return to standby by hardware reset. The only one parameter is a 
 *         check code, the command would be executed if check code = 0xA5. 
 *         You can use Epd::Reset() to awaken and use Epd::Init() to initialize.
 */
void Epd::Sleep() {
//  SendCommand(POWER_OFF);
  WaitUntilIdle();
  SendCommand(DEEP_SLEEP);
  SendData(0xa5);
}




//***************lut_GC(void)************************//	
//full refresh waveform GC 
//***************************************************//
void Epd::lut_GC(void)
{
	unsigned int count;	 
	
		SendCommand(0x20);							//vcom
		for(count=0;count<56;count++)
			{SendData(lut_R20_GC[count]);}
		
	  SendCommand(0x21);							//red not use
	  for(count=0;count<56;count++)
		  {SendData(lut_R21_GC[count]);}

		SendCommand(0x24);							//bb b
		for(count=0;count<56;count++)
			{SendData(lut_R24_GC[count]);}			
			


	  SendCommand(0x22);							//bw r
		for(count=0;count<56;count++)
			{SendData(lut_R22_GC[count]);}

		SendCommand(0x23);							//wb w
		for(count=0;count<56;count++)
			{SendData(lut_R23_GC[count]);}
 

	
	 			         
}


//***************lut_DU(void)************************//	
//Partially refresh waveform
//***************************************************//
void Epd::lut_DU(void)
{
	unsigned int count;	 
	
		SendCommand(0x20);							//vcom
		for(count=0;count<56;count++)
			{SendData(lut_R20_DU[count]);}
		
	  SendCommand(0x21);							//red not use
	  for(count=0;count<56;count++)
		  {SendData(lut_R21_DU[count]);}

		SendCommand(0x24);							//bb b
		for(count=0;count<56;count++)
			{SendData(lut_R24_DU[count]);}			
			

	  SendCommand(0x22);							//bw r
		for(count=0;count<56;count++)
			{SendData(lut_R22_DU[count]);}

		SendCommand(0x23);							//wb w
		for(count=0;count<56;count++)
			{SendData(lut_R23_DU[count]);}

	
	 			         
}






//////////////////////////////////////full screen update LUT////////////////////////////////////////////

//Full refresh waveform, refresh time 800ms
const unsigned char  lut_R20_GC[] =  
{
0x01,	0x00,	0x14,	0x14,	0x01,	0x01,	0x00,	0x01,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};							  
const unsigned char    lut_R21_GC[] =  
{
0x01,	0x60,	0x14,	0x14,	0x01,	0x01,	0x00,	0x01,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};						 
const unsigned char   lut_R22_GC[] =  
{
0x01,	0x60,	0x14,	0x14,	0x01,	0x01,	0x00,	0x01,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};
const unsigned char  lut_R23_GC[] =  
{
0x01,	0x90,	0x14,	0x14,	0x01,	0x01,	0x00,	0x01,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};
const unsigned char   lut_R24_GC[] =  
{
0x01,	0x90,	0x14,	0x14,	0x01,	0x01,	0x00,	0x01,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};


// Partially refresh waveform, screen refresh time 400ms
const unsigned char  lut_R20_DU[] =  
{
0x01,	0x00,	0x14,	0x01,	0x01,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};							  
const unsigned char   lut_R21_DU[] =  
{
0x01,	0x20,	0x14,	0x01,	0x01,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};						 
const unsigned char   lut_R22_DU[] =  
{
0x01,	0x80,	0x14,	0x01,	0x01,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};
const unsigned char   lut_R23_DU[] =  
{
0x01,	0x40,	0x14,	0x01,	0x01,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};
const unsigned char  lut_R24_DU[] =  
{
0x01,	0x00,	0x14,	0x01,	0x01,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,	0x00,
};


/* END OF FILE */


